/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.usv.wrapper;

import java.util.Date;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import de.bsvrz.sys.usv.enums.BatterieZustand;
import de.bsvrz.sys.usv.enums.EingangsfrequenzZustand;
import de.bsvrz.sys.usv.enums.KommunikationsZustand;
import de.bsvrz.sys.usv.enums.KritischerZustand;
import de.bsvrz.sys.usv.enums.USVZustand;
import de.bsvrz.sys.usv.enums.WechselSpannung;
import de.bsvrz.sys.usv.enums.ZustandNotstrom;
import de.bsvrz.sys.usv.enums.ZustandPhase;

/**
 * Klasse die die Kommunikation mit der Klasse durchführt, die eine reale
 * USV-Anlage anbindet. Die Klasse benutzt dabei die Methoden, die im Interface
 * USVAllgemein beschrieben sind. Die Methode welche Klasse die reale USV
 * anbindet wird dabei als Aufrufparameter übergeben.
 *
 * @author ObertM Geprüft 30.9.2008 Hilpp
 * @author Christian Hösel, BitCtrl Systems GmbH
 */
public class SenderThread extends Thread implements ClientSenderInterface {

	// ---------------------------------------------------------------------------
	// Definitionen für die Texte, die in den einzelnen Attributgruppen
	// verwendet werden.
	// ---------------------------------------------------------------------------
	private final USVAllgemein m_usv;
	private final String m_pid;
	private final int m_zyklusZeit;

	// ---------------------------------------------------------------------------
	// Attribute einer USV
	// ---------------------------------------------------------------------------
	private int m_batterieStatus = 0;
	private USVZustand m_usvZustand = USVZustand.na;
	private KommunikationsZustand m_kommunikationsZustand = KommunikationsZustand.na;
	private EingangsfrequenzZustand m_eingangsFrequenzZustand = EingangsfrequenzZustand.na;
	private WechselSpannung m_wechselSpannung = WechselSpannung.na;
	private ZustandPhase m_zustandPhase1 = ZustandPhase.na;
	private ZustandPhase m_zustandPhase2 = ZustandPhase.na;
	private ZustandPhase m_zustandPhase3 = ZustandPhase.na;
	private ZustandNotstrom m_zustandNotstrom = ZustandNotstrom.na;
	private float batterieTemperatur = -1f;
	private int batterieRestlaufzeit = -1;
	private int batterieBetriebsdauer = -1;
	private int batterieSpannung = -1;

	private int m_oldBatterieStatus = BatterieZustand.na.getValue();
	private USVZustand m_oldUsvStatus = USVZustand.na;
	private KommunikationsZustand m_oldKommunikationsZustand = KommunikationsZustand.na;
	private EingangsfrequenzZustand m_oldEingangsFrequenzZustand = EingangsfrequenzZustand.na;
	private WechselSpannung m_oldWechselSpannung = WechselSpannung.na;
	private ZustandPhase m_oldZustandPhase1 = ZustandPhase.na;
	private ZustandPhase m_oldZustandPhase2 = ZustandPhase.na;
	private ZustandPhase m_oldZustandPhase3 = ZustandPhase.na;
	private ZustandNotstrom m_oldZustandNotstrom = ZustandNotstrom.na;
	private float oldBatterieTemperatur = -1f;
	private int oldBatterieRestlaufzeit = -1;
	private int oldBatterieBetriebsdauer = -1;
	private int oldBatterieSpannung = -1;

	private KritischerZustand m_oldKritischerZustand = KritischerZustand.na;

	/**
	 * Flags, die anzeigen, ob beim nächsten USV-Datenabruf anschließend das
	 * Ergebnis an den DAV versendet werden soll, da beim letzten Mal kein
	 * erfolgreiches Versenden möglich war - sprich beim letzten Mal senden kam
	 * ein Exception, also versuchen wir es nochmal.
	 */
	private boolean usvZustandNochmalversenden = false;
	private boolean usvKritischerZustandNochmalversenden = false;

	/**
	 * Unsere Datenverteilerverbindung.
	 */
	final private ClientDavInterface conSenden;

	/**
	 * Attributgruppe zur Darstellung der Zustandsattribute
	 */
	private final AttributeGroup atg_usvZustand;

	/**
	 * Aspekt: "asp.zustand";
	 */
	private final Aspect asp_zustand;

	/**
	 * Das USV-Dav-Objekt.
	 */
	private final SystemObject usvSend;

	/**
	 * Attributgruppe, die angibt, ob ein kritischer Zustand erreicht wurde oder
	 * nicht.
	 */
	private final AttributeGroup atg_usvZustandKritisch;

	/**
	 * Der Dav-Logger.
	 */
	private final Debug log = Debug.getLogger();

	private final String schwellWertBedingung;

	/**
	 * Konstruktor der Klasse
	 *
	 * @param usv
	 *            Klasse die das Interface USVAllgemein implementiert
	 * @param pid
	 *            Pid der realen USV
	 * @param schwellenWert
	 *            die Schwellenwertbedingung, bei der, wenn sie erfüllt ist, der
	 *            DAV heruntegefahren wird.
	 * @param zyklusZeit
	 *            das Zeitintervall, in dem eine Aktualisierung der USV-Daten
	 *            vorgenommen wird.
	 */
	public SenderThread(final USVAllgemein usv, final String pid, final String schwellenWert, final int zyklusZeit) {

		// Setzen der Membervariablen der Klasse
		m_usv = usv;
		m_pid = pid;
		m_zyklusZeit = zyklusZeit;
		schwellWertBedingung = schwellenWert;

		// Sende Funktion
		conSenden = USVWrapper.getConnection();

		// ---------------------------------------------------------------------------
		// Anmelden am DAV für das Senden der Attributgruppe usvStatus und
		// versenden eines initialen Datensatzen mit "nicht ermittelbar"
		// ---------------------------------------------------------------------------
		atg_usvZustand = conSenden.getDataModel().getAttributeGroup(UsvKonstanten.atg_usvZustand);
		asp_zustand = conSenden.getDataModel().getAspect(UsvKonstanten.asp_zustand);
		usvSend = conSenden.getDataModel().getObject(m_pid);

		try {
			final ResultData result = new ResultData(usvSend, new DataDescription(atg_usvZustand, asp_zustand),
					conSenden.getTime(), erzeugeDefaultUSVZustandsDaten());
			conSenden.subscribeSource(this, result);
		} catch (final OneSubscriptionPerSendData e1) {
			throw new IllegalStateException("Es konnte keine korrekte Anmeldung als Quelle für " + atg_usvZustand
					+ " mit " + asp_zustand + " für das USV Objekt " + usvSend.getNameOrPidOrId() + " erfolgen.", e1);
		}

		// ---------------------------------------------------------------------------
		// Anmelden am DAV für das Senden der Attributgruppe usvZustandKritisch
		// und versenden eines initialen Datensatzes mit "nicht ermittelbar"
		// ---------------------------------------------------------------------------
		atg_usvZustandKritisch = conSenden.getDataModel().getAttributeGroup(UsvKonstanten.atg_usvZustandKritisch);
		try {
			final Data newDatenUsvZustandKritisch = conSenden.createData(atg_usvZustandKritisch);
			newDatenUsvZustandKritisch.getScaledValue(UsvKonstanten.kritischerZustand)
			.setText(KritischerZustand.na.getText());
			final ResultData resultData = new ResultData(usvSend,
					new DataDescription(atg_usvZustandKritisch, asp_zustand), conSenden.getTime(),
					newDatenUsvZustandKritisch);
			conSenden.subscribeSource(this, resultData);
		} catch (final OneSubscriptionPerSendData e1) {
			throw new IllegalStateException(
					"Es konnte keine korrekte Anmeldung als Quelle für " + atg_usvZustandKritisch + " mit "
							+ asp_zustand + " für das USV Objekt " + usvSend.getNameOrPidOrId() + " erfolgen.",
							e1);
		}

	}

	/**
	 * Erzeugt einen default-Datensatz für die Attributgruppe "atg.usvZustand"
	 * wobei alle Attribute auf "nicht ermittelbar" (-1) gesetzt werden.
	 *
	 * @return den Standard-Datensatz für die Attributgruppe "atg.usvZustand".
	 */
	private Data erzeugeDefaultUSVZustandsDaten() {
		final Data result = conSenden.createData(atg_usvZustand);

		result.getScaledValue(UsvKonstanten.usvZustand).setText(USVZustand.na.getText());
		result.getScaledValue(UsvKonstanten.batterieZustand).setText(BatterieZustand.na.getText());
		result.getScaledValue(UsvKonstanten.kommunikationsZustand).setText(KommunikationsZustand.na.getText());
		result.getScaledValue(UsvKonstanten.eingangsFrequenzZustand).setText(EingangsfrequenzZustand.na.getText());
		result.getScaledValue(UsvKonstanten.wechselSpannung).setText(WechselSpannung.na.getText());
		result.getScaledValue(UsvKonstanten.zustandPhase1).setText(ZustandPhase.na.getText());
		result.getScaledValue(UsvKonstanten.zustandPhase2).setText(ZustandPhase.na.getText());
		result.getScaledValue(UsvKonstanten.zustandPhase3).setText(ZustandPhase.na.getText());
		result.getScaledValue(UsvKonstanten.zustandNotStrom).setText(ZustandNotstrom.na.getText());

		// TODO:Wenn der Datk. fertig ist, dann geht das vielleicht auch!!!!
		/*
		 * result.getScaledValue(UsvKonstanten.batterieBetriebsdauer).setText(
		 * UsvKonstanten.nicht_ermittelbar);
		 * result.getScaledValue(UsvKonstanten.batterieRestlaufzeit).setText(
		 * UsvKonstanten.nicht_ermittelbar);
		 * result.getScaledValue(UsvKonstanten.batterieSpannung).setText(
		 * UsvKonstanten.nicht_ermittelbar);
		 * result.getScaledValue(UsvKonstanten.batterieTemperatur).setText(
		 * UsvKonstanten.nicht_ermittelbar);
		 */
		return result;
	}

	/**
	 * Thread der die Kommunikation mit der Klasse, die die reale USV-Anlage
	 * einbindet, durchführt. Dabei werden zyklisch die Daten der USV-Anlage
	 * eingelesen und bei Änderung an den Datenverteiler gesendet. Die
	 * Zykluszeit kann durch die Methoden die im Interface USVTrigger definiert
	 * sind unterbrochen werden.
	 */
	@Override
	public synchronized void run() {

		// ---------------------------------------------------------------------------
		// Initialisieren der Attributgruppe usvStatus
		// ---------------------------------------------------------------------------
		final Data newDataUsvZustand = erzeugeDefaultUSVZustandsDaten();

		// ---------------------------------------------------------------------------
		// Initialisieren der Attributgruppe usvZustandKritisch
		// ---------------------------------------------------------------------------
		final Data newDatenUsvZustandKritisch = conSenden.createData(atg_usvZustandKritisch);
		newDatenUsvZustandKritisch.getScaledValue(UsvKonstanten.kritischerZustand)
		.setText(KritischerZustand.na.getText());

		// ---------------------------------------------------------------------------
		// In einem Endlosloop wird zyklisch der Status der USV eingelesen
		// Bei Änderung in den Daten wird der geänderte Status publiziert.
		// Bei Erreichen des kritischen Punktes wird dieser publiziert.
		// ---------------------------------------------------------------------------
		while (true) {

			log.info("Starte neuen USV Abfragezyklus...");

			// ---------------------------------------------------------------------------
			// Einlesen der aktuellen Werte
			// ---------------------------------------------------------------------------
			m_batterieStatus = m_usv.getBatterieZustand();
			m_usvZustand = m_usv.getUsvZustand();
			m_kommunikationsZustand = m_usv.getKommunikationsZustand();
			m_eingangsFrequenzZustand = m_usv.getEingangsFrequenzZustand();
			m_wechselSpannung = m_usv.getWechselSpannung();
			m_zustandPhase1 = m_usv.getZustandPhase1();
			m_zustandPhase2 = m_usv.getZustandPhase2();
			m_zustandPhase3 = m_usv.getZustandPhase3();
			m_zustandNotstrom = m_usv.getZustandNotstrom();
			batterieSpannung = m_usv.getBatterieSpannung();
			batterieTemperatur = m_usv.getBatterieTemperatur();
			batterieBetriebsdauer = m_usv.getBetriebsdauerBatterie();
			batterieRestlaufzeit = m_usv.getUSVRestlaufzeit();

			// ------------------------------------------------------------------------
			// Publizieren des USV-Status bei Änderung oder, wenn beim letzten
			// mal senden
			// irgendwas nicht funktioniert hat.
			// ------------------------------------------------------------------------
			if (pruefeAenderungZustandUsv() || usvZustandNochmalversenden || usvKritischerZustandNochmalversenden) {
				try {
					newDataUsvZustand.getScaledValue(UsvKonstanten.usvZustand).setText(m_usvZustand.getText());

					final BatterieZustand bStatus = BatterieZustand.findBatterieZustand(m_batterieStatus);
					if (bStatus != null) {
						newDataUsvZustand.getScaledValue(UsvKonstanten.batterieZustand).setText(bStatus.getText());
					} else {
						newDataUsvZustand.getScaledValue(UsvKonstanten.batterieZustand).set(m_batterieStatus);
					}

					newDataUsvZustand.getScaledValue(UsvKonstanten.kommunikationsZustand)
					.setText(m_kommunikationsZustand.getText());
					newDataUsvZustand.getScaledValue(UsvKonstanten.eingangsFrequenzZustand)
					.setText(m_eingangsFrequenzZustand.getText());
					newDataUsvZustand.getScaledValue(UsvKonstanten.wechselSpannung)
					.setText(m_wechselSpannung.getText());
					newDataUsvZustand.getScaledValue(UsvKonstanten.zustandPhase1).setText(m_zustandPhase1.getText());
					newDataUsvZustand.getScaledValue(UsvKonstanten.zustandPhase2).setText(m_zustandPhase2.getText());
					newDataUsvZustand.getScaledValue(UsvKonstanten.zustandPhase3).setText(m_zustandPhase3.getText());
					newDataUsvZustand.getScaledValue(UsvKonstanten.zustandNotStrom)
					.setText(m_zustandNotstrom.getText());

					// TODO:Wenn der Datk. fertig ist, dann geht das vielleicht
					// auch!!!!

					/*
					 * if (batterieBetriebsdauer < 0) {
					 * newDataUsvZustand.getScaledValue(
					 * UsvKonstanten.batterieBetriebsdauer).setText(
					 * UsvKonstanten.nicht_ermittelbar); } else {
					 * newDataUsvZustand.getScaledValue(
					 * UsvKonstanten.batterieBetriebsdauer).set(
					 * batterieBetriebsdauer); }
					 *
					 * if (batterieRestlaufzeit < 0) {
					 * newDataUsvZustand.getScaledValue(
					 * UsvKonstanten.batterieRestlaufzeit).setText(
					 * UsvKonstanten.nicht_ermittelbar); } else {
					 * newDataUsvZustand.getScaledValue(
					 * UsvKonstanten.batterieRestlaufzeit).set(
					 * batterieRestlaufzeit); }
					 *
					 * if (batterieSpannung < 0) {
					 * newDataUsvZustand.getScaledValue(
					 * UsvKonstanten.batterieSpannung).setText(
					 * UsvKonstanten.nicht_ermittelbar); } else {
					 * newDataUsvZustand.getScaledValue(
					 * UsvKonstanten.batterieSpannung).set( batterieSpannung); }
					 *
					 * if (batterieTemperatur < 0) {
					 * newDataUsvZustand.getScaledValue(
					 * UsvKonstanten.batterieTemperatur).setText(
					 * UsvKonstanten.nicht_ermittelbar); } else {
					 * newDataUsvZustand.getScaledValue(
					 * UsvKonstanten.batterieTemperatur).set(
					 * batterieTemperatur); }
					 */

					final ResultData rsd1 = new ResultData(usvSend, new DataDescription(atg_usvZustand, asp_zustand),
							new Date().getTime(), newDataUsvZustand);
					conSenden.sendData(rsd1);
					usvZustandNochmalversenden = false;
				} catch (final SendSubscriptionNotConfirmed e1) {
					log.warning("Neuer Datensatz kann NOCH nicht verschickt werden. ", e1);
					usvZustandNochmalversenden = true;
				}

				// ------------------------------------------------------------------------
				// Auswerten, ob kritischer Punkt erreicht wurde
				// ------------------------------------------------------------------------
				if (schwellWertBedingung.length() <= 0) {
					log.warning("Keine Prüfung kritischer Schwellwerte, da keine Parameter übergeben wurde.");
				} else {
					KritischerZustand kritischerZustand = KritischerZustand.na; // Default

					/**
					 * XXX:Wenn der Betriebszustand der USV ungewiss ("nicht
					 * ermittelbar") ist, dann kann auch kein KritscherZustand
					 * ermittelt werden --> kritischerZustand ist dann auch
					 * "nicht ermittelbar".
					 */
					if (!m_usvZustand.equals(USVZustand.na)) {

						// TODO: wieder alte Methode benutzen (nach
						// Datenkatalogaenderung)
						// final boolean ergebnis = SchwellwertParser
						// .auswertung(
						// schwellWertBedingung,
						// newDataUsvZustand);

						final boolean ergebnis = SchwellwertParser.auswertungMitRestLaufZeit(schwellWertBedingung,
								newDataUsvZustand, batterieRestlaufzeit);

						if (ergebnis) {
							kritischerZustand = KritischerZustand.kritisch;
						} else {
							kritischerZustand = KritischerZustand.normal;
						}
					}
					// ------------------------------------------------------------------------
					// Kritischer Zustand hat sich geändert
					// ------------------------------------------------------------------------
					if (!m_oldKritischerZustand.equals(kritischerZustand)) {
						m_oldKritischerZustand = kritischerZustand;

						// ----------------------------------------------------------------------
						// Betriebsmeldung absetzen
						// ----------------------------------------------------------------------
						final String text = "USV-kritischer Zustand: " + kritischerZustand;

						log.info("Versende neu Betriebsmeldung mit folgendem Inhalt: " + text);

						final MessageType typ = MessageType.SYSTEM_DOMAIN;
						final String messageTypeAddOn = "Wechsel";
						final MessageGrade grade = MessageGrade.INFORMATION;
						final String message = text;

						sendMessage(typ, messageTypeAddOn, grade, message);

						try {
							newDatenUsvZustandKritisch.getScaledValue(UsvKonstanten.kritischerZustand)
							.setText(kritischerZustand.getText());

							final ResultData rsd2 = new ResultData(usvSend,
									new DataDescription(atg_usvZustandKritisch, asp_zustand), new Date().getTime(),
									newDatenUsvZustandKritisch);

							conSenden.sendData(rsd2);
							usvKritischerZustandNochmalversenden = false;
						} catch (final SendSubscriptionNotConfirmed e1) {
							log.warning("Neuer Datensatz kann NOCH nicht verschickt werden. ", e1);
							usvKritischerZustandNochmalversenden = true;
						}
					}
				}
			}

			// Alle m_zyklusZeit Sekunden durchlaufen oder bei Änderung
			try {
				synchronized (this) {
					wait(m_zyklusZeit * 1000);
				}
			} catch (final InterruptedException e) {
				// Warten abgebrochen
			}

		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @seestauma.dav.clientside.ClientSenderInterface#dataRequest(stauma.dav.
	 * configuration.interfaces.SystemObject,
	 * stauma.dav.clientside.DataDescription, byte)
	 */
	@Override
	public void dataRequest(final SystemObject arg0, final DataDescription arg1, final byte arg2) {
		System.out.println("DataRequest: " + arg2);
		// brauchen wir nicht, wir senden wann es uns passt
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * stauma.dav.clientside.ClientSenderInterface#isRequestSupported(stauma
	 * .dav.configuration.interfaces.SystemObject,
	 * stauma.dav.clientside.DataDescription)
	 */
	@Override
	public boolean isRequestSupported(final SystemObject arg0, final DataDescription arg1) {
		return true;
	}

	/**
	 * Methode zum Unterbrechen der wait-Zyklen im Sender Thread.
	 */
	public synchronized void setTrigger() {
		notify();
	}

	/**
	 * Methode prüft ob sich die Werte der USV-Anlage geändert haben, wenn ja,
	 * dann werden die Änderungen protokolliert
	 *
	 * @return 0: keine Änderung, &gt;0 Anzahl der geänderten Werte
	 */
	private boolean pruefeAenderungZustandUsv() {
		String s1;
		String s2;

		int aenderung = 0;

		if (m_batterieStatus != m_oldBatterieStatus) {
			final BatterieZustand bStatus = BatterieZustand.findBatterieZustand(m_batterieStatus);
			if (bStatus != null) {
				s1 = bStatus.getText();
			} else {
				s1 = m_batterieStatus + " %";
			}

			final BatterieZustand oldBStatus = BatterieZustand.findBatterieZustand(m_oldBatterieStatus);
			if (oldBStatus != null) {
				s2 = oldBStatus.getText();
			} else {
				s2 = m_oldBatterieStatus + " %";
			}

			protokolliereZustandUsv(UsvKonstanten.batterieZustand, s1, s2);

			aenderung++;
			m_oldBatterieStatus = m_batterieStatus;
		}

		if (!m_usvZustand.equals(m_oldUsvStatus)) {
			protokolliereZustandUsv(UsvKonstanten.usvZustand, m_usvZustand.getText(), m_oldUsvStatus.getText());

			aenderung++;
			m_oldUsvStatus = m_usvZustand;
		}

		if (!m_kommunikationsZustand.equals(m_oldKommunikationsZustand)) {
			protokolliereZustandUsv(UsvKonstanten.kommunikationsZustand, m_kommunikationsZustand.getText(),
					m_oldKommunikationsZustand.getText());
			aenderung++;
			m_oldKommunikationsZustand = m_kommunikationsZustand;
		}

		if (!m_eingangsFrequenzZustand.equals(m_oldEingangsFrequenzZustand)) {
			protokolliereZustandUsv(UsvKonstanten.eingangsFrequenzZustand, m_eingangsFrequenzZustand.getText(),
					m_oldEingangsFrequenzZustand.getText());

			aenderung++;
			m_oldEingangsFrequenzZustand = m_eingangsFrequenzZustand;
		}

		if (!m_wechselSpannung.equals(m_oldWechselSpannung)) {
			protokolliereZustandUsv(UsvKonstanten.wechselSpannung, m_wechselSpannung.getText(),
					m_oldWechselSpannung.getText());
			aenderung++;
			m_oldWechselSpannung = m_wechselSpannung;
		}

		if (!m_zustandPhase1.equals(m_oldZustandPhase1)) {
			protokolliereZustandUsv(UsvKonstanten.zustandPhase1, m_zustandPhase1.getText(),
					m_oldZustandPhase1.getText());
			aenderung++;
			m_oldZustandPhase1 = m_zustandPhase1;
		}

		if (!m_zustandPhase2.equals(m_oldZustandPhase2)) {
			protokolliereZustandUsv(UsvKonstanten.zustandPhase2, m_zustandPhase2.getText(),
					m_oldZustandPhase2.getText());

			aenderung++;
			m_oldZustandPhase2 = m_zustandPhase2;
		}

		if (!m_zustandPhase3.equals(m_oldZustandPhase3)) {
			protokolliereZustandUsv(UsvKonstanten.zustandPhase3, m_zustandPhase3.getText(),
					m_oldZustandPhase3.getText());
			aenderung++;
			m_oldZustandPhase3 = m_zustandPhase3;
		}

		if (!m_zustandNotstrom.equals(m_oldZustandNotstrom)) {
			protokolliereZustandUsv(UsvKonstanten.zustandNotStrom, m_zustandNotstrom.getText(),
					m_oldZustandNotstrom.getText());
			aenderung++;
			m_oldZustandNotstrom = m_zustandNotstrom;
		}

		if (batterieSpannung != oldBatterieSpannung) {
			protokolliereZustandUsv(UsvKonstanten.batterieSpannung, Integer.toString(batterieSpannung),
					Integer.toString(oldBatterieSpannung));
			aenderung++;
			oldBatterieSpannung = batterieSpannung;
		}

		if (batterieTemperatur != batterieTemperatur) {
			protokolliereZustandUsv(UsvKonstanten.batterieTemperatur, Float.toString(batterieTemperatur),
					Float.toString(oldBatterieTemperatur));
			aenderung++;
			oldBatterieTemperatur = batterieTemperatur;
		}

		if (batterieBetriebsdauer != oldBatterieBetriebsdauer) {
			protokolliereZustandUsv(UsvKonstanten.batterieBetriebsdauer, Integer.toString(batterieBetriebsdauer),
					Integer.toString(oldBatterieBetriebsdauer));
			aenderung++;
			oldBatterieBetriebsdauer = batterieBetriebsdauer;
		}

		if (batterieRestlaufzeit != oldBatterieRestlaufzeit) {
			protokolliereZustandUsv(UsvKonstanten.batterieRestlaufzeit, Integer.toString(batterieRestlaufzeit),
					Integer.toString(oldBatterieRestlaufzeit));
			aenderung++;
			oldBatterieRestlaufzeit = batterieRestlaufzeit;
		}

		return aenderung != 0;
	}

	/**
	 * Methode zum Absetzen einer Betriebsmeldung an den Datenverteiler
	 *
	 * @param objekt
	 *            Name des sich änderenden Objekts
	 * @param zustandNeu
	 *            Neuer Zustand des Objekts
	 * @param zustandAlt
	 *            Alter Zustand des Objekts
	 */
	private void protokolliereZustandUsv(final String objekt, final String zustandNeu, final String zustandAlt) {
		final String text = objekt + " Wechsel von \"" + zustandAlt + "\" nach \"" + zustandNeu + "\"";

		Debug.getLogger().info(text);

		final MessageType typ = MessageType.SYSTEM_DOMAIN;
		final String messageTypeAddOn = "Wechsel";
		final MessageGrade grade = MessageGrade.INFORMATION;
		final String message = text;

		sendMessage(typ, messageTypeAddOn, grade, message);
	}

	/**
	 * Methode erzeugt eine Betriebsmeldung und sendet diese an den
	 * Datenverteiler. Eingestellt werden können MeldungsTyp, MeldungsTypZusatz
	 * und die Meldungsklasse.
	 *
	 * @param type
	 *            der MeldungsTyp
	 * @param messageTypeAddOn
	 *            der MeldungsTypZusatz
	 * @param grade
	 *            die MeldungsKlasse
	 * @param message
	 *            Text der Meldung
	 */
	public void sendMessage(final MessageType type, final String messageTypeAddOn, final MessageGrade grade,
			final String message) {
		final MessageSender msg = MessageSender.getInstance();

		if (msg == null) {
			log.error(
					"Es können kein Betriebsmeldungen abgesetzt werden, MessageSender = null. Folgende Meldung sollte versendet werden: "
							+ message);
			return;
		}

		msg.setApplicationLabel(USVWrapper.getPidUsv());

		final String id = "" + USVWrapper.getConnection().getLocalApplicationObject().getId();

		final SystemObject referenceObject = usvSend;
		final MessageState state = MessageState.CHANGE_MESSAGE;
		final MessageCauser causer = new MessageCauser(null, "", "");

		msg.sendMessage(id, type, messageTypeAddOn, grade, referenceObject, state, causer, message);
	}
}
