/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.usv.wrapper;

import de.bsvrz.sys.usv.enums.EingangsfrequenzZustand;
import de.bsvrz.sys.usv.enums.KommunikationsZustand;
import de.bsvrz.sys.usv.enums.USVZustand;
import de.bsvrz.sys.usv.enums.WechselSpannung;
import de.bsvrz.sys.usv.enums.ZustandNotstrom;
import de.bsvrz.sys.usv.enums.ZustandPhase;

/**
 * Interface zur Anbindung einer realen USV-Anlage. Über dieses Interface
 * liefert das Modul, das eine reale USV-Anlage anbindet die Informationen
 * dieser Anlage zurück, bzw. nimmt Schaltbefehle für diese Anlage entgegen.
 * 
 * @author ObertM Geprüft 30.9.2008 Hilpp
 * @author BitCtrl Systems GmbH, Christian Hösel
 */
public interface USVAllgemein {

	/**
	 * Default-Timeout in Millisekunden, der für die Kommunikation mit der USV
	 * verwandt wird. Via Kommandozeilen-Parameter, kann dieser Wert verändert
	 * werden. In der Praxis kann es nämlich sein, dass die Latentzeiten in
	 * Netzwerken erheblich variieren.
	 */
	public static final long DEFAULT_TIMEOUT = 5000L;

	/**
	 * Liefert den aktuellen Zustand einer USV
	 * 
	 * @return ein USV-Zustands-Enum.
	 */
	public abstract USVZustand getUsvZustand();

	/**
	 * Liefert den aktuellen Batteriezustand
	 * 
	 * @return -1 : nicht ermittelbar 0-100 : Ladezustand der Batterie in %
	 */
	public abstract int getBatterieZustand();

	/**
	 * Liefert den aktuellen Kommunikationszustand
	 * 
	 * @return ein Kommunikationszustands-Enum.
	 */
	public abstract KommunikationsZustand getKommunikationsZustand();

	/**
	 * Liefert den aktuellen Zustand der Eingangsfrequenz
	 * 
	 * @return ein Eingangsfrequenzzustands-Enum.
	 */
	public abstract EingangsfrequenzZustand getEingangsFrequenzZustand();

	/**
	 * Liefert den aktuellen Zustand der Wechselspannung
	 * 
	 * @return ein Wechselspannungs-Enum.
	 */
	public abstract WechselSpannung getWechselSpannung();

	/**
	 * Liefert den aktuellen Zustand der Phase 1
	 * 
	 * @return Enum, das den Zustand der Phase1 wiederspiegelt.
	 */
	public abstract ZustandPhase getZustandPhase1();

	/**
	 * Liefert den aktuellen Zustand der Phase 2
	 * 
	 * @return Enum, das den Zustand der Phase2 wiederspiegelt.
	 */
	public abstract ZustandPhase getZustandPhase2();

	/**
	 * Liefert den aktuellen Zustand der Phase 3
	 * 
	 * @return Enum, das den Zustand der Phase3 wiederspiegelt.
	 */
	public abstract ZustandPhase getZustandPhase3();

	/**
	 * Liefert den aktuellen Zustand der Notstromversorgung
	 * 
	 * @return Zustand der Notstromversorgung.
	 */
	public abstract ZustandNotstrom getZustandNotstrom();

	/**
	 * Methode schaltet ein Notstromaggregat ein
	 * 
	 * @param befehl
	 *            0: passiv 1: aktiv
	 * @return -1: Funktionalität nicht unterstützt 0: Funktion erfolgreich
	 *         durchgeführt 1: Fehler beim Ausführen der Schaltung
	 */
	public abstract int setNotstrom(boolean befehl);

	/**
	 * Methode schaltet den USV-Betrieb ein
	 * 
	 * @param befehl
	 *            0: Normalbetrieb 1: Batteriebetrieb
	 * @return -1: Funktionalität nicht unterstützt 0: Funktion erfolgreich
	 *         durchgeführt 1: Fehler beim Ausführen der Schaltung
	 */
	public abstract int setUsvBetrieb(boolean befehl);

	/**
	 * Methode mit der der Klasse, die das Interface USVAllgemein implementiert
	 * die Instanz der Hauptklasse mitgeteilt mit. Mit dieser Instanz kann das
	 * Interface USVTrigger benutzt werden
	 * 
	 * @param instance
	 *            Instanz der Hauptklasse
	 */
	public abstract void setInstance(USVWrapper instance);

	/**
	 * Liefert die Temperatur der Batterie der USV in °C.
	 * 
	 * @return die Temperatur der Batterie der USV
	 */
	public int getBatterieTemperatur();

	/**
	 * Liefert die Restlaufzeit der USV im Batteriebetrieb in Minuten.
	 * 
	 * @return die Restlaufzeit der USV, oder -1 für "nicht ermittelbar".
	 */
	public int getUSVRestlaufzeit();

	/**
	 * Liefert die aktuelle Betriebsdauer der USV im Batteriebetrieb in
	 * Sekunden.
	 * 
	 * @return die Betriebsdauer der USV im Batteriebetrieb, oder -1 für "nicht
	 *         ermittelbar".
	 */
	public int getBetriebsdauerBatterie();

	/**
	 * Liefert die Batteriespannung der USV.
	 * 
	 * @return die Batteriespannung der USV, oder -1 für "nicht ermittelbar".
	 */
	public int getBatterieSpannung();

}
